--
-- Events Module
--
-- Frames deal with their own events (mostly)
--

-- Import
local AH = Crafty_AH
local AHFrameList = Crafty_AHFrameList
local Data = Crafty_Data
local QueueFrame = Crafty_QueueFrame
local ScanTip = Crafty_ScanningTooltip
local ShowUIPanel = ShowUIPanel
local TradeSkill = Crafty_TradeSkill
local CreateFrame = CreateFrame
local type = type

-- Make sure we don't polute the global environment
setfenv(1, {})

local function HandleEvent(self, event, ...)
    --Data:Debug("HandleEvent", event)
    if event == "PLAYER_ENTERING_WORLD" then
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	Data:OnLoad()
	AH:OnLoad()
	ScanTip:Create()
	TradeSkill:OnLoad()
    elseif event == "TRADE_SKILL_SHOW" then
	self:UnregisterEvent("TRADE_SKILL_SHOW")
	TradeSkill:OnOpen()
	-- Don't open Queue when AH is open as there is no room and causes a bug
	if AH.open then
	    -- NOOP
	else
	    ShowUIPanel(QueueFrame)
	end
	self:RegisterEvent("TRADE_SKILL_LIST_UPDATE")
	self:RegisterEvent("TRADE_SKILL_CLOSE")
    elseif event == "TRADE_SKILL_LIST_UPDATE" then
	if TradeSkill.open then
	    TradeSkill:DetectChange()
	    if TradeSkill.scan then
		TradeSkill:Scan()
	    end
	end
    elseif event == "TRADE_SKILL_CLOSE" then
	self:UnregisterEvent("TRADE_SKILL_CLOSE")
	self:UnregisterEvent("TRADE_SKILL_LIST_UPDATE")
	TradeSkill:OnClose()
	self:RegisterEvent("TRADE_SKILL_SHOW")
    elseif event == "AUCTION_HOUSE_SHOW" then
	self:UnregisterEvent("AUCTION_HOUSE_SHOW")
	AH:OnOpen()
	self:RegisterEvent("AUCTION_ITEM_LIST_UPDATE")
	self:RegisterEvent("AUCTION_HOUSE_CLOSED")
    elseif event == "AUCTION_ITEM_LIST_UPDATE" then
	if AH.open and AH.scanning.query then
	    AH:OnListUpdate()
	end
    elseif event == "AUCTION_HOUSE_CLOSED" then
	self:UnregisterEvent("AUCTION_HOUSE_CLOSED")
	self:UnregisterEvent("AUCTION_ITEM_LIST_UPDATE")
	-- Can fire twice
	if AH.open then 
	    AH:OnClose()
	    AHFrameList:OnClose()
	end
	self:RegisterEvent("AUCTION_HOUSE_SHOW")
    end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("TRADE_SKILL_SHOW")
--frame:RegisterEvent("NEW_RECIPE_LEARNED") -- arg1 = recipeID
frame:RegisterEvent("AUCTION_HOUSE_SHOW")
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:SetScript("OnEvent", HandleEvent)
